unit VaxCallSessionRoute;

interface

uses
    System.StrUtils,
    System.SysUtils,
    System.Classes,
    VaxServerCOM,
    Generics.Collections;

type TUserRouteDetails = class

public
    m_sUserPrefix, m_sToPeerName, m_sDialPrefix: String;
    m_nToPeerType: Integer;

end;

type TLineRouteDetails = class

  public

    m_sLineName, m_sRoomName, m_sToPeerName: String;
    m_nToPeerType: Integer;
    m_bRejected: Boolean;

end;

type TDirectProxyRouteDetails = class

  public

    m_sProxyName, m_sRoomName, m_sToPeerName: String;
    m_nToPeerType: Integer;
    m_bRejected: Boolean;

end;

type TVaxCallSessionRoute = class

    var
        m_objMapUserRouteToPtr: TObjectDictionary<string, TUserRouteDetails>;
        m_objMapLineRouteToPtr: TObjectDictionary<string, TLineRouteDetails>;
        m_objMapDirectProxyRouteToPtr: TObjectDictionary<string, TDirectProxyRouteDetails>;

    private

        function GenKey(sValue: String): String;

        function FindUserRoute(sUserName: String): TUserRouteDetails;
        function FindLineRoute(sLineName: String): TLineRouteDetails;
        function FindDirectProxyRoute(sProxyName: String): TDirectProxyRouteDetails;

        procedure DeleteAllUserRoutes();
        procedure DeleteAllLineRoutes();
        procedure DeleteAllDirectProxyRoutes();

        function GetUserRouteDetails(sUserName: String; var sUserPrefix, sDialPrefix: String; var nToPeerType: Integer; var sToPeerName: String): Boolean;
        function GetLineRouteDetails(sLineName: String; var nToPeerType: Integer; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String): Boolean;
        function GetProxyRouteDetails(sProxyName: String; var nToPeerType: Integer; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String): Boolean;

    public

        constructor Create();
        Destructor Destroy();

        procedure ResetOBJ();

        function AddUserRoute(sUserName, sUserPrefix, sDialPrefix: String; nToPeerType: Integer; sToPeerName: String): Boolean;
        procedure RemoveUserRoute(sUserName: String);

        function AddLineRoute(sLineName: String; nToPeerType: Integer; sToPeerName: String; bRejected: Boolean; sRoomName: String): Boolean;
        procedure RemoveLineRoute(sLineName: String);

        function AddDirectProxyRoute(sProxyName: String; nToPeerType: Integer; sToPeerName: String; bRejected: Boolean; sRoomName: String): Boolean;
        procedure RemoveDirectProxyRoute(sProxyName: String);

        function GetUserRouteInfo(sUserName: String; sUserDialedNo: String; var sDialNo: String; var nToPeerType: Integer; var sToPeerName: String): Boolean;
        function GetLineRouteInfo(sLineName: String; var nToPeerType: Integer; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String): Boolean;
        function GetProxyRouteInfo(sProxyName: String; var nToPeerType: Integer; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String): Boolean;

end;

implementation

        constructor TVaxCallSessionRoute.Create();
        begin
            m_objMapUserRouteToPtr := TObjectDictionary<string, TUserRouteDetails>.Create([doOwnsValues], 256);
            m_objMapLineRouteToPtr := TObjectDictionary<string, TLineRouteDetails>.Create([doOwnsValues], 256);
            m_objMapDirectProxyRouteToPtr := TObjectDictionary<string, TDirectProxyRouteDetails>.Create([doOwnsValues], 256);
        end;

        Destructor TVaxCallSessionRoute.Destroy();
        begin
            DeleteAllUserRoutes();
            DeleteAllLineRoutes();
            DeleteAllDirectProxyRoutes();
        end;

        function TVaxCallSessionRoute.GenKey(sValue: String): String;
        begin
            Result := AnsiLowerCase(sValue);
        end;

        function TVaxCallSessionRoute.FindUserRoute(sUserName: String): TUserRouteDetails;
        var
            bResult: Boolean;
            objUserRoute: TUserRouteDetails;
        begin
            bResult := m_objMapUserRouteToPtr.TryGetValue(AnsiLowerCase(sUserName), objUserRoute);
            if(bResult = False) then Exit(nil);

            Result := objUserRoute;
        end;

        function TVaxCallSessionRoute.FindLineRoute(sLineName: String): TLineRouteDetails;
        var
            bResult: Boolean;
            objLineRoute: TLineRouteDetails;
        begin
            bResult := m_objMapLineRouteToPtr.TryGetValue(AnsiLowerCase(sLineName), objLineRoute);
            if(bResult = False) then Exit(nil);

            Result := objLineRoute;
        end;

        function TVaxCallSessionRoute.FindDirectProxyRoute(sProxyName: String): TDirectProxyRouteDetails;
        var
            bResult: Boolean;
            objProxyRoute: TDirectProxyRouteDetails;
        begin
            bResult := m_objMapDirectProxyRouteToPtr.TryGetValue(AnsiLowerCase(sProxyName), objProxyRoute);
            if(bResult = False) then Exit(nil);

            Result := objProxyRoute;
        end;

        procedure TVaxCallSessionRoute.DeleteAllUserRoutes();
        var
            objUserRoute: TUserRouteDetails;
        begin

            if(m_objMapUserRouteToPtr.Count <= 0) then
              Exit;

            for objUserRoute in m_objMapUserRouteToPtr.Values do
              begin
                    objUserRoute.Free;
              end;

              if(m_objMapUserRouteToPtr.Count > 0) then
                  m_objMapUserRouteToPtr.Clear();

        end;

        procedure TVaxCallSessionRoute.DeleteAllLineRoutes();
        var
            objLineRoute: TLineRouteDetails;
        begin

            if(m_objMapLineRouteToPtr.Count <= 0) then
              Exit;

            for objLineRoute in m_objMapLineRouteToPtr.Values do
              begin
                    objLineRoute.Free;
              end;

              if(m_objMapLineRouteToPtr.Count > 0 ) then
                    m_objMapLineRouteToPtr.Clear();
        end;

        procedure TVaxCallSessionRoute.DeleteAllDirectProxyRoutes();
        var
            objProxyRoute: TDirectProxyRouteDetails;
        begin

            if(m_objMapDirectProxyRouteToPtr.Count <= 0) then
              Exit;

            for objProxyRoute in m_objMapDirectProxyRouteToPtr.Values do
              begin
                    objProxyRoute.Free;
              end;

              if(m_objMapDirectProxyRouteToPtr.Count > 0 ) then
                  m_objMapDirectProxyRouteToPtr.Clear();
        end;

        function TVaxCallSessionRoute.GetUserRouteDetails(sUserName: String; var sUserPrefix, sDialPrefix: String; var nToPeerType: Integer; var sToPeerName: String): Boolean;
        var
            objUserRoute: TUserRouteDetails;
        begin

            objUserRoute := FindUserRoute(sUserName);
            if(objUserRoute = nil) then Exit(False);

            sUserPrefix := objUserRoute.m_sUserPrefix;
            sDialPrefix := objUserRoute.m_sDialPrefix;

            nToPeerType := objUserRoute.m_nToPeerType;
            sToPeerName := objUserRoute.m_sToPeerName;

            Exit(True);

        end;

        function TVaxCallSessionRoute.GetLineRouteDetails(sLineName: String; var nToPeerType: Integer; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String): Boolean;
        var
            objLineRoute: TLineRouteDetails;
        begin

            objLineRoute := FindLineRoute(sLineName);
            if(objLineRoute = nil) then Exit(False);

            nToPeerType := objLineRoute.m_nToPeerType;
            sToPeerName := objLineRoute.m_sToPeerName;

            bRejected := objLineRoute.m_bRejected;
            sRoomName := objLineRoute.m_sRoomName;

            Exit(True);

        end;

        function TVaxCallSessionRoute.GetProxyRouteDetails(sProxyName: String; var nToPeerType: Integer; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String): Boolean;
        var
            objProxyRoute: TDirectProxyRouteDetails;
        begin

            objProxyRoute := FindDirectProxyRoute(sProxyName);
            if(objProxyRoute = nil) then Exit(False);

            nToPeerType := objProxyRoute.m_nToPeerType;
            sToPeerName := objProxyRoute.m_sToPeerName;

            bRejected := objProxyRoute.m_bRejected;
            sRoomName := objProxyRoute.m_sRoomName;

            Exit(True);

        end;

        procedure TVaxCallSessionRoute.ResetOBJ();
        begin
            DeleteAllUserRoutes();
            DeleteAllLineRoutes();
            DeleteAllDirectProxyRoutes();
        end;

        function TVaxCallSessionRoute.AddUserRoute(sUserName, sUserPrefix, sDialPrefix: String; nToPeerType: Integer; sToPeerName: String): Boolean;        var
            sKey: String;
            objUserRoute: TUserRouteDetails;
        begin

            sKey := GenKey(sUserName);

            objUserRoute := FindUserRoute(sUserName);
            if(objUserRoute = nil) then
            begin

                objUserRoute := TUserRouteDetails.Create();

                objUserRoute.m_sUserPrefix := sUserPrefix;
                objUserRoute.m_sDialPrefix := sDialPrefix;

                objUserRoute.m_nToPeerType := nToPeerType;
                objUserRoute.m_sToPeerName := sToPeerName;

                m_objMapUserRouteToPtr.Add(sKey, objUserRoute);

                Exit(True);
            end;

            objUserRoute.m_sUserPrefix := sUserPrefix;
            objUserRoute.m_sDialPrefix := sDialPrefix;

            objUserRoute.m_nToPeerType := nToPeerType;
            objUserRoute.m_sToPeerName := sToPeerName;

            Exit(True);
        end;

        procedure TVaxCallSessionRoute.RemoveUserRoute(sUserName: String);
        var
            objUserRoute: TUserRouteDetails;
        begin
            objUserRoute := FindUserRoute(sUserName);
            if(objUserRoute = nil) then Exit;

            m_objMapUserRouteToPtr.Remove(AnsiLowerCase(sUserName));
            //FreeAndNil(objUserRoute);
        end;

        function TVaxCallSessionRoute.AddLineRoute(sLineName: String; nToPeerType: Integer; sToPeerName: String; bRejected: Boolean; sRoomName: String): Boolean;
        var
            sKey: String;
            objLineRoute: TLineRouteDetails;
        begin

            sKey := GenKey(sLineName);

            objLineRoute := FindLineRoute(sLineName);
            if(objLineRoute = nil) then
            begin
                objLineRoute := TLineRouteDetails.Create();

                objLineRoute.m_nToPeerType := nToPeerType;
                objLineRoute.m_sToPeerName := sToPeerName;

                objLineRoute.m_bRejected := bRejected;
                objLineRoute.m_sRoomName := sRoomName;

                m_objMapLineRouteToPtr.Add(sKey, objLineRoute);

                Exit(True);
            end;

            objLineRoute.m_nToPeerType := nToPeerType;
            objLineRoute.m_sToPeerName := sToPeerName;

            objLineRoute.m_bRejected := bRejected;
            objLineRoute.m_sRoomName := sRoomName;

            Exit(True);
        end;

        procedure TVaxCallSessionRoute.RemoveLineRoute(sLineName: String);
        var
            objLineRoute: TLineRouteDetails;
        begin

            objLineRoute := FindLineRoute(sLineName);
            if(objLineRoute = nil) then Exit;

            m_objMapLineRouteToPtr.Remove(AnsiLowerCase(sLineName));
            //FreeAndNil(objLineRoute);

        end;

        function TVaxCallSessionRoute.AddDirectProxyRoute(sProxyName: String; nToPeerType: Integer; sToPeerName: String; bRejected: Boolean; sRoomName: String): Boolean;
        var
            sKey: String;
            objProxyRoute: TDirectProxyRouteDetails;
        begin

            sKey := GenKey(sProxyName);

            objProxyRoute := FindDirectProxyRoute(sProxyName);
            if(objProxyRoute = nil) then
            begin

                objProxyRoute := TDirectProxyRouteDetails.Create();

                objProxyRoute.m_nToPeerType := nToPeerType;
                objProxyRoute.m_sToPeerName := sToPeerName;

                objProxyRoute.m_bRejected := bRejected;
                objProxyRoute.m_sRoomName := sRoomName;

                m_objMapDirectProxyRouteToPtr.Add(sKey, objProxyRoute);

                Exit(True);

            end;

            objProxyRoute.m_nToPeerType := nToPeerType;
            objProxyRoute.m_sToPeerName := sToPeerName;

            objProxyRoute.m_bRejected := bRejected;
            objProxyRoute.m_sRoomName := sRoomName;

            // Reference of object. No need to call following function
            //m_objMapDirectProxyRouteToPtr.AddOrSetValue(sKey, objProxyRoute);

            Result := True;

        end;

        procedure TVaxCallSessionRoute.RemoveDirectProxyRoute(sProxyName: String);
        var
            objProxyRoute: TDirectProxyRouteDetails;
        begin

            objProxyRoute := FindDirectProxyRoute(sProxyName);
            if(objProxyRoute = nil) then Exit;

            m_objMapDirectProxyRouteToPtr.Remove(AnsiLowerCase(sProxyName));
            //FreeAndNil(objProxyRoute);
        end;

        function TVaxCallSessionRoute.GetUserRouteInfo(sUserName: String; sUserDialedNo: String; var sDialNo: String; var nToPeerType: Integer; var sToPeerName: String): Boolean;
        var
            nCharCount, nToPeerNameLen: Integer;
            sUserPrefix, sDialPrefix: String;
            bResult: Boolean;
        begin

            bResult := GetUserRouteDetails(sUserName, sUserPrefix, sDialPrefix, nToPeerType, sToPeerName);
            nToPeerNameLen := Length(sToPeerName);

            if ((bResult = False) OR (nToPeerType = -1) OR (nToPeerNameLen <= 0)) then
            begin

              sDialNo := sUserDialedNo;
              nToPeerType := TVaxServerCOM.VAX_PEER_TYPE_USER;
              sToPeerName := sUserDialedNo;

              Exit(True);

            end;

            nCharCount := Length(sUserPrefix);

            if(AnsiMidStr(sUserDialedNo, 1, nCharCount) = sUserPrefix) then // see if 'sUserDialNo' has the 'sUserPrefix' ?
            begin
                Delete(sUserDialedNo, 1, nCharCount);
                sDialNo := sDialPrefix + sUserDialedNo;
            end
            else // else make it USER_TO_USER call
            begin
              sDialNo := sUserDialedNo;
              nToPeerType := TVaxServerCOM.VAX_PEER_TYPE_USER;
              sToPeerName := sUserDialedNo;
            end;

            Result := True;

        end;

        function TVaxCallSessionRoute.GetLineRouteInfo(sLineName: String; var nToPeerType: Integer; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String): Boolean;
        begin
            Result := GetLineRouteDetails(sLineName, nToPeerType, sToPeerName, bRejected, sRoomName);
        end;

        function TVaxCallSessionRoute.GetProxyRouteInfo(sProxyName: String; var nToPeerType: Integer; var sToPeerName: String; var bRejected: Boolean; var sRoomName: String): Boolean;
        begin
            Result := GetProxyRouteDetails(sProxyName, nToPeerType, sToPeerName, bRejected, sRoomName);
        end;


end.
