#include "StdAfx.h"
#include "StoreRegistry.h"
#include "StoreCOMM.h"

#define SUB_KEY_BUFFER_SIZE	2000

#define ROOT_KEY_PATH "Software\\VaxTeleServerSIP"

CStoreRegistry::CStoreRegistry()
{
	Open();
}

CStoreRegistry::~CStoreRegistry()
{
	Close();
}

///////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////

BOOL CStoreRegistry::Open()
{
	if(::RegCreateKeyEx(HKEY_CURRENT_USER, ROOT_KEY_PATH, 0, NULL, 0, KEY_ALL_ACCESS, NULL, &m_hKey, NULL) != ERROR_SUCCESS)
	{
		m_hKey = NULL;
		return FALSE;
	}
	
	return TRUE;
}

void CStoreRegistry::Close()
{
	if(m_hKey == NULL)
		return;
	
	::RegCloseKey(m_hKey);
	m_hKey = NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////

HKEY CStoreRegistry::CreateKey(CString sKeyName)
{
	if(m_hKey == NULL)
		return FALSE;

	if(sKeyName.IsEmpty())
		return NULL;

	HKEY hKeySub = NULL;
	if(::RegCreateKeyEx(m_hKey, sKeyName, 0, NULL, 0, KEY_ALL_ACCESS, NULL, &hKeySub, NULL) != ERROR_SUCCESS)
		return NULL;

	return hKeySub;
}

HKEY CStoreRegistry::OpenKey(CString sKeyName)
{
	if(m_hKey == NULL)
		return FALSE;

	if(sKeyName.IsEmpty())
		return NULL;

	HKEY hKeySub = NULL;
	if(::RegOpenKeyEx(m_hKey, sKeyName, 0, KEY_ALL_ACCESS, &hKeySub) != ERROR_SUCCESS)
		return FALSE;

	return hKeySub;
}

////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////

BOOL CStoreRegistry::SetTEXT(LPCTSTR sKeyName, LPCTSTR sValueName, LPCTSTR sValueData)
{
	HKEY hKey = CreateKey(sKeyName);
	if(hKey == NULL)	return FALSE;

	if(::RegSetValueEx(hKey, sValueName, 0 , REG_SZ, (BYTE*) sValueData, ::lstrlen(sValueData)) != ERROR_SUCCESS)
		return FALSE;

	return TRUE;
}

BOOL CStoreRegistry::SetDWORD(LPCTSTR sKeyName, LPCTSTR sValueName, DWORD dwValueData)
{
	HKEY hKey = CreateKey(sKeyName);
	if(hKey == NULL)	return FALSE;

	if(::RegSetValueEx(hKey, sValueName, 0, REG_DWORD, (const BYTE*)&dwValueData, sizeof(dwValueData)))
		return FALSE;

	return TRUE;
}

BOOL CStoreRegistry::SetINT(LPCTSTR sKeyName, LPCTSTR sValueName, int nValueData)
{
	HKEY hKey = CreateKey(sKeyName);
    if(hKey == NULL) return FALSE;

	if(::RegSetValueEx(hKey, sValueName, 0, REG_DWORD , (const BYTE*)&nValueData, sizeof(nValueData)))
		return FALSE;

	return TRUE;	
}

BOOL CStoreRegistry::SetBOOL(LPCTSTR sKeyName, LPCTSTR sValueName, BOOL bValueData)
{
	int nValueData = 0;
	if(bValueData == TRUE)	nValueData = 1;

	if(!SetINT(sKeyName, sValueName, nValueData))
		return FALSE;

	return TRUE;
}

BOOL CStoreRegistry::GetTEXT(LPCTSTR sKeyName, LPCTSTR sValueName, CString& sValueData)
{
	sValueData = "";

	HKEY hKey = OpenKey(sKeyName);
    
	if(hKey == NULL)
	   return FALSE;
	
	char arrBuffer[SUB_KEY_BUFFER_SIZE] = "\0";
	DWORD dwCount = SUB_KEY_BUFFER_SIZE;

	if(::RegQueryValueEx(hKey, sValueName, NULL, NULL, (BYTE*)arrBuffer, &dwCount) != ERROR_SUCCESS)
		return FALSE;

	sValueData = arrBuffer;
	return TRUE;
}

BOOL CStoreRegistry::GetDWORD(LPCTSTR sKeyName, LPCTSTR sValueName, DWORD& dwValueData)
{
	HKEY hKey = OpenKey(sKeyName);
    
	if(hKey == NULL)
	   return FALSE;
	
	char arrBuffer[SUB_KEY_BUFFER_SIZE] = "\0";
	DWORD dwCount = SUB_KEY_BUFFER_SIZE;

	if(::RegQueryValueEx(hKey, sValueName, NULL, NULL, (BYTE*)arrBuffer, &dwCount) != ERROR_SUCCESS)
		return FALSE;

	dwValueData = *((DWORD*) arrBuffer);

	return TRUE;
}

BOOL CStoreRegistry::GetINT(LPCTSTR sKeyName, LPCTSTR sValueName, int& nValueData)
{
   HKEY hKey = OpenKey(sKeyName);

	if(hKey == NULL)
	   return FALSE;
	
	char arrBuffer[SUB_KEY_BUFFER_SIZE] = "\0";
	DWORD dwCount = SUB_KEY_BUFFER_SIZE;

	if(::RegQueryValueEx(hKey, sValueName, NULL, NULL, (BYTE*)arrBuffer, &dwCount) != ERROR_SUCCESS)
		return FALSE;

    nValueData = *((int*) arrBuffer);
	
	return TRUE;
}

BOOL CStoreRegistry::GetBOOL(LPCTSTR sKeyName, LPCTSTR sValueName, BOOL& bValueData)
{
	int nValueData = 0;
	bValueData = FALSE;

	if(!GetINT(sKeyName, sValueName, nValueData))
		return FALSE;

	if(nValueData == 1)
		bValueData = TRUE;
	
	return TRUE;
}

BOOL CStoreRegistry::DeleteSubKey(LPCTSTR sKeyName)
{
	if(CString(sKeyName).IsEmpty())
		return FALSE;

	::RegDeleteTree(m_hKey, sKeyName);
	return TRUE;
}

BOOL CStoreRegistry::EnumSubKeys(LPCTSTR sKeyRootName, CStringList* objListSubKey)
{
	HKEY hKey = OpenKey(sKeyRootName);

	if(hKey == NULL)
		return FALSE;
		 
	char arraySubKey[SUB_KEY_BUFFER_SIZE] = "\0";
	DWORD dwSize = SUB_KEY_BUFFER_SIZE;

	for(int nCount = 0;;nCount++)
	{
		if(::RegEnumKeyEx(hKey, nCount, arraySubKey, &dwSize, 0, NULL, NULL, NULL)  != ERROR_SUCCESS)
			break;
		
		objListSubKey->AddTail(arraySubKey);

		::memset(arraySubKey, 0, SUB_KEY_BUFFER_SIZE);
		dwSize = SUB_KEY_BUFFER_SIZE;
	}

	::RegCloseKey(hKey);
	
	return TRUE;
}

BOOL CStoreRegistry::DeleteValue(LPCTSTR sKeyRootName, LPCTSTR sValueName)
{
	HKEY hKey = OpenKey(sKeyRootName);

	if(hKey == NULL)
		return FALSE;

	::RegDeleteKey(hKey, sValueName);
	::RegCloseKey(hKey);

	 return TRUE;
}

BOOL CStoreRegistry::IsKeyExist(LPCTSTR sSubKeyName)
{
	HKEY hKey = OpenKey(sSubKeyName);

	if(hKey == NULL)
		return FALSE;

	::RegCloseKey(hKey);
	return TRUE;
}